@@ -33,7 +33,7 @@ CREATE_SUBNET_RESPONSE = """
   <subnet>
     <subnetId>{{ subnet.id }}</subnetId>
     <state>pending</state>
-    <vpcId>{{ subnet.vpc.id }}</vpcId>
+    <vpcId>{{ subnet.vpc_id }}</vpcId>
     <cidrBlock>{{ subnet.cidr_block }}</cidrBlock>
     <availableIpAddressCount>251</availableIpAddressCount>
     <availabilityZone>us-east-1a</availabilityZone>
@@ -55,7 +55,7 @@ DESCRIBE_SUBNETS_RESPONSE = """
       <item>
         <subnetId>{{ subnet.id }}</subnetId>
         <state>available</state>
-        <vpcId>{{ subnet.vpc.id }}</vpcId>
+        <vpcId>{{ subnet.vpc_id }}</vpcId>
         <cidrBlock>{{ subnet.cidr_block }}</cidrBlock>
         <availableIpAddressCount>251</availableIpAddressCount>
         <availabilityZone>us-east-1a</availabilityZone>
