@@ -1,10 +1,11 @@
 from jinja2 import Template
 
+from moto.core.responses import BaseResponse
 from moto.ec2.models import ec2_backend
 from moto.ec2.utils import resource_ids_from_querystring
 
 
-class TagResponse(object):
+class TagResponse(BaseResponse):
 
     def create_tags(self):
         resource_ids = resource_ids_from_querystring(self.querystring)
