@@ -2,34 +2,57 @@ import os
 import base64
 import datetime
 import hashlib
+import copy
 
 from moto.core import BaseBackend
 from moto.core.utils import iso_8601_datetime, rfc_1123_datetime
+from .exceptions import BucketAlreadyExists
 from .utils import clean_key_name
 
-UPLOAD_ID_BYTES=43
-UPLOAD_PART_MIN_SIZE=5242880
+UPLOAD_ID_BYTES = 43
+UPLOAD_PART_MIN_SIZE = 5242880
 
 
 class FakeKey(object):
-    def __init__(self, name, value):
+    def __init__(self, name, value, storage="STANDARD", etag=None):
         self.name = name
         self.value = value
         self.last_modified = datetime.datetime.now()
+        self._storage_class = storage
         self._metadata = {}
+        self._expiry = None
+        self._etag = etag
+
+    def copy(self, new_name=None):
+        r = copy.deepcopy(self)
+        if new_name is not None:
+            r.name = new_name
+        return r
 
     def set_metadata(self, key, metadata):
         self._metadata[key] = metadata
 
+    def clear_metadata(self):
+        self._metadata = {}
+
+    def set_storage_class(self, storage_class):
+        self._storage_class = storage_class
+
     def append_to_value(self, value):
         self.value += value
         self.last_modified = datetime.datetime.now()
+        self._etag = None  # must recalculate etag
+
+    def restore(self, days):
+        self._expiry = datetime.datetime.now() + datetime.timedelta(days)
 
     @property
     def etag(self):
-        value_md5 = hashlib.md5()
-        value_md5.update(bytes(self.value))
-        return '"{0}"'.format(value_md5.hexdigest())
+        if self._etag is None:
+            value_md5 = hashlib.md5()
+            value_md5.update(bytes(self.value))
+            self._etag = value_md5.hexdigest()
+        return '"{0}"'.format(self._etag)
 
     @property
     def last_modified_ISO8601(self):
@@ -47,15 +70,30 @@ class FakeKey(object):
 
     @property
     def response_dict(self):
-        return {
+        r = {
             'etag': self.etag,
             'last-modified': self.last_modified_RFC1123,
         }
+        if self._storage_class != 'STANDARD':
+            r['x-amz-storage-class'] = self._storage_class
+        if self._expiry is not None:
+            rhdr = 'ongoing-request="false", expiry-date="{0}"'
+            r['x-amz-restore'] = rhdr.format(self.expiry_date)
+        return r
 
     @property
     def size(self):
         return len(self.value)
 
+    @property
+    def storage_class(self):
+        return self._storage_class
+
+    @property
+    def expiry_date(self):
+        if self._expiry is not None:
+            return self._expiry.strftime("%a, %d %b %Y %H:%M:%S GMT")
+
 
 class FakeMultipart(object):
     def __init__(self, key_name):
@@ -65,14 +103,18 @@ class FakeMultipart(object):
 
     def complete(self):
         total = bytearray()
+        md5s = bytearray()
         last_part_name = len(self.list_parts())
 
         for part in self.list_parts():
             if part.name != last_part_name and len(part.value) < UPLOAD_PART_MIN_SIZE:
-                return
+                return None, None
+            md5s.extend(part.etag.replace('"', '').decode('hex'))
             total.extend(part.value)
 
-        return total
+        etag = hashlib.md5()
+        etag.update(bytes(md5s))
+        return total, "{0}-{1}".format(etag.hexdigest(), last_part_name)
 
     def set_part(self, part_id, value):
         if part_id < 1:
@@ -107,6 +149,8 @@ class S3Backend(BaseBackend):
         self.buckets = {}
 
     def create_bucket(self, bucket_name):
+        if bucket_name in self.buckets:
+            raise BucketAlreadyExists()
         new_bucket = FakeBucket(name=bucket_name)
         self.buckets[bucket_name] = new_bucket
         return new_bucket
@@ -127,11 +171,12 @@ class S3Backend(BaseBackend):
                 return self.buckets.pop(bucket_name)
         return None
 
-    def set_key(self, bucket_name, key_name, value):
+    def set_key(self, bucket_name, key_name, value, storage=None, etag=None):
         key_name = clean_key_name(key_name)
 
         bucket = self.buckets[bucket_name]
-        new_key = FakeKey(name=key_name, value=value)
+        new_key = FakeKey(name=key_name, value=value,
+                          storage=storage, etag=etag)
         bucket.keys[key_name] = new_key
 
         return new_key
@@ -159,12 +204,12 @@ class S3Backend(BaseBackend):
     def complete_multipart(self, bucket_name, multipart_id):
         bucket = self.buckets[bucket_name]
         multipart = bucket.multiparts[multipart_id]
-        value = multipart.complete()
+        value, etag = multipart.complete()
         if value is None:
             return
         del bucket.multiparts[multipart_id]
 
-        return self.set_key(bucket_name, multipart.key_name, value)
+        return self.set_key(bucket_name, multipart.key_name, value, etag=etag)
 
     def cancel_multipart(self, bucket_name, multipart_id):
         bucket = self.buckets[bucket_name]
@@ -174,11 +219,23 @@ class S3Backend(BaseBackend):
         bucket = self.buckets[bucket_name]
         return bucket.multiparts[multipart_id].list_parts()
 
+    def get_all_multiparts(self, bucket_name):
+        bucket = self.buckets[bucket_name]
+        return bucket.multiparts
+
     def set_part(self, bucket_name, multipart_id, part_id, value):
         bucket = self.buckets[bucket_name]
         multipart = bucket.multiparts[multipart_id]
         return multipart.set_part(part_id, value)
 
+    def copy_part(self, dest_bucket_name, multipart_id, part_id,
+                  src_bucket_name, src_key_name):
+        src_key_name = clean_key_name(src_key_name)
+        src_bucket = self.buckets[src_bucket_name]
+        dest_bucket = self.buckets[dest_bucket_name]
+        multipart = dest_bucket.multiparts[multipart_id]
+        return multipart.set_part(part_id, src_bucket.keys[src_key_name].value)
+
     def prefix_query(self, bucket, prefix, delimiter):
         key_results = set()
         folder_results = set()
@@ -210,11 +267,16 @@ class S3Backend(BaseBackend):
         bucket = self.buckets[bucket_name]
         return bucket.keys.pop(key_name)
 
-    def copy_key(self, src_bucket_name, src_key_name, dest_bucket_name, dest_key_name):
+    def copy_key(self, src_bucket_name, src_key_name, dest_bucket_name, dest_key_name, storage=None):
         src_key_name = clean_key_name(src_key_name)
         dest_key_name = clean_key_name(dest_key_name)
         src_bucket = self.buckets[src_bucket_name]
         dest_bucket = self.buckets[dest_bucket_name]
-        dest_bucket.keys[dest_key_name] = src_bucket.keys[src_key_name]
+        key = src_bucket.keys[src_key_name]
+        if dest_key_name != src_key_name:
+            key = key.copy(dest_key_name)
+        dest_bucket.keys[dest_key_name] = key
+        if storage is not None:
+            dest_bucket.keys[dest_key_name].set_storage_class(storage)
 
 s3_backend = S3Backend()
