@@ -16,5 +16,5 @@ url_paths = {
     '{0}/$': bucket_response3,
     '{0}/(?P<bucket_name>[a-zA-Z0-9\-_.]+)$': ro.bucket_response,
     '{0}/(?P<bucket_name>[a-zA-Z0-9\-_.]+)/$': bucket_response2,
-    '{0}/(?P<bucket_name>[a-zA-Z0-9\-_./]+)/(?P<key_name>[a-zA-Z0-9\-_.?]+)': ro.key_response
+    '{0}/(?P<bucket_name>[a-zA-Z0-9\-_./]+)/(?P<key_name>.+)': ro.key_response
 }
