@@ -51,6 +51,19 @@ class Queue(object):
         self.queue_arn = 'arn:aws:sqs:sqs.us-east-1:123456789012:%s' % self.name
         self.receive_message_wait_time_seconds = 0
 
+    @classmethod
+    def create_from_cloudformation_json(cls, resource_name, cloudformation_json):
+        properties = cloudformation_json['Properties']
+
+        return sqs_backend.create_queue(
+            name=properties['QueueName'],
+            visibility_timeout=properties.get('VisibilityTimeout'),
+        )
+
+    @property
+    def physical_resource_id(self):
+        return self.name
+
     @property
     def attributes(self):
         result = {}
