@@ -4,9 +4,11 @@ from moto.core.utils import iso_8601_datetime
 
 
 class Token(object):
-    def __init__(self, duration):
+    def __init__(self, duration, name=None, policy=None):
         now = datetime.datetime.now()
         self.expiration = now + datetime.timedelta(seconds=duration)
+        self.name = name
+        self.policy = None
 
     @property
     def expiration_ISO8601(self):
@@ -32,6 +34,10 @@ class STSBackend(BaseBackend):
         token = Token(duration=duration)
         return token
 
+    def get_federation_token(self, name, duration, policy):
+        token = Token(duration=duration, name=name, policy=policy)
+        return token
+
     def assume_role(self, **kwargs):
         role = AssumedRole(**kwargs)
         return role
